/***************************************************************************
                          main.cc - Fax Front End for Hylafax
                             -------------------
    begin                : Thu Jan 8, 2004
    copyright            : (C) 2004 by Glenn Burkhardt
    email                : gbburkhardt@aaahawk.com

    Usage:  faxfrontend [file to fax]

            If invoked with no arguments, user is allowed to enter
            fax file or send cover page only.

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <syslog.h>
#include <pwd.h>
#include <qapplication.h>
#include <qfileinfo.h>
#include <qfont.h>
#include <qstring.h>
#include <qtextcodec.h>
#include <qtranslator.h>
#include <qcheckbox.h>
#include <qcombobox.h>
#include <qmessagebox.h>
#include <qbuttongroup.h>
#include <qmultilineedit.h>
#include <qspinbox.h>
#include <qlineedit.h>
#include <qplatinumstyle.h>
#include "FaxFrontEnd.h"
#include "faxstat.h"

int main(int ac, char *av[])
{
    char tmpname[] = "/tmp/FaxCoverXXXXXX";
    char *user, *faxfile;
    bool delete_tmpname=false, debug=false;
    int status=0;
    struct passwd *pw;

    QApplication a(ac, av);
    a.setStyle(new QPlatinumStyle);

    QFileInfo fi(av[0]);
    
    QTranslator tor( 0 );
    // set the location where your .qm files are in load() below as the 
    // last parameter instead of "."
    // for development, use "/" to use the english original as
    // .qm files are stored in the base project directory.
    tor.load( QString("faxfrontend.")
	      + QTextCodec::locale(), fi.dirPath(true) );
    a.installTranslator( &tor );

    /* Check for arguments: -d for debug, file to fax */
    faxfile = 0;
    for (int i=1; i<ac; i++) {
	if (av[i][0] == '-' && av[i][1] == 'd') {
	    debug = true;
	    openlog("faxfrontend", 0, LOG_DAEMON);
	} else
	    faxfile = av[i];
    }

    FaxFrontEnd faxfrontend(faxfile);
    faxfrontend.show();
    
    // send the fax
    if (faxfrontend.exec() == QDialog::Accepted) {
	QString cmd;

	cmd = "sendfax";
  	
  	if (faxfrontend.noCoverPage->isChecked()) {
	    cmd += " -n";
  	} else {
	    if (!faxfrontend.companyBox->text().isEmpty())
		cmd += " -x '"
		    + faxfrontend.companyBox->text().stripWhiteSpace() + "'";
	    if (!faxfrontend.locationBox->text().isEmpty())
		cmd += " -y '"
		    + faxfrontend.locationBox->text().stripWhiteSpace() + "'";
	    if (!faxfrontend.regardingBox->text().isEmpty())
		cmd += " -r '"
		    + faxfrontend.regardingBox->text().stripWhiteSpace() + "'";
	    if (!faxfrontend.comments->text().isEmpty())
		cmd += " -c '"
		    + faxfrontend.comments->text().stripWhiteSpace() + "'";
  	}
  	
  	if (faxfrontend.notifyComplete->isChecked())
	    cmd += " -D";
  	else
	    cmd += " -N";
  	if (faxfrontend.notifyRetry->isChecked()) cmd += " -R";
  	if (faxfrontend.resButtonGroup->selected()
	    == (QButton*)faxfrontend.lowResButton) cmd += " -l";
  	
  	// transmit delay value
  	if (faxfrontend.xmitSpinBox->value() > 0) {
	    cmd += QString(" -a 'now + %1 minutes'")
		.arg(faxfrontend.xmitSpinBox->value());
  	}
  	if (faxfrontend.retrySpinBox->value() > 1) {
	    QString retry;
	    retry.sprintf(" -t %d", faxfrontend.retrySpinBox->value());
	    cmd += retry;
  	}
  	if (faxfrontend.pageSizeComboBox->currentItem())
	    cmd += " -s a4";
  	else
	    cmd += " -s na-let";
  	
  	cmd += QString(" -d '%1@%2'").arg(faxfrontend.recipientBox->text().stripWhiteSpace())
	    .arg(faxfrontend.faxNumberBox->text().stripWhiteSpace());

  	cmd += " ";	// to precede filename

	/* If no arguments were given, assume test mode */
	if (faxfile)
	    cmd += faxfile;	// fax file name
	else if (!faxfrontend.fileLineEdit->text().isEmpty())
	    cmd += faxfrontend.fileLineEdit->text();
	else {
	    /* Send cover page only */
	    if (QMessageBox::warning(0, QObject::tr("Warning"),
				     QObject::tr("No file to fax specified.\n"
						 "Send Cover Page only?"),
				     QMessageBox::Yes|QMessageBox::Default,
				     QMessageBox::No) == QMessageBox::No) {
		return 0;
	    }

	    mktemp(tmpname);
	    QString fcmd("faxcover");

	    fcmd += " -t '"
		+ faxfrontend.recipientBox->text().stripWhiteSpace() + "'";

	    if (!faxfrontend.companyBox->text().isEmpty())
		fcmd += " -x '"
		    + faxfrontend.companyBox->text().stripWhiteSpace() + "'";
	    if (!faxfrontend.locationBox->text().isEmpty())
		fcmd += " -l '"
		    + faxfrontend.locationBox->text().stripWhiteSpace() + "'";
	    if (!faxfrontend.regardingBox->text().isEmpty())
		fcmd += " -r '"
		    + faxfrontend.regardingBox->text().stripWhiteSpace() + "'";
	    if (!faxfrontend.comments->text().isEmpty())
		fcmd += " -c '"
		    + faxfrontend.comments->text().stripWhiteSpace() + "'";

	    fcmd += " -p 0";

	    /* Get 'real user' name for "from" address */
	    user = getenv("USER");
	    if (!user) user = "root";
	    pw = getpwnam(user);
	    if (!pw) 
		fcmd += " -f HylaFAX";
	    else {		
		fcmd += " -f '";
		fcmd += pw->pw_gecos;
		fcmd += "'";
	    }

	    fcmd += " -n '"
		+ faxfrontend.faxNumberBox->text().stripWhiteSpace() + "'";

	    fcmd += " > ";
	    fcmd += tmpname;

	    if (debug) {
		syslog(LOG_INFO, fcmd.latin1());
	    }

	    status = system(fcmd.latin1());
	    status = WEXITSTATUS(status);
	    if (status) {
		QMessageBox::critical(0, QObject::tr("Error"),
			      QObject::tr("Can't create faxcover\ncommand: %1")
				      .arg(fcmd));
	    }

	    delete_tmpname = true;

	    cmd += tmpname;
	}

	if (!status) {

	    if (debug) {
		syslog(LOG_INFO, cmd.latin1());
	    }

	    status = system(cmd.latin1());	// send the fax
	    status = WEXITSTATUS(status);
	    if (status) {
		QMessageBox::critical(0, QObject::tr("Error"),
			      QObject::tr("Can't send fax\ncommand: %1")
				      .arg(cmd));
	    }

	    if (delete_tmpname) unlink(tmpname);

	    // Now poll status from Hylafax
	    FaxStat fs;
	    fs.show();
	    fs.exec();
	}
    }

    return status;
}


